/********************************************************************
   T/SDAS-DDC1 DCM Demo Utility. DAC-1/2 and DIO demo for C6x. rev.1A
   (C) MicroLAB Systems, 1999


   Description:
   ------------
	This demo writes SIN waves with different amplitudes and
	frequencies to DAC-1/2.

	DIO-0 and DIO-1 outputs are al;tered every sampling period with
	DIO-1 being an inverse state of DIO-0.

	Test patterns are being transmited over XSL-1 and XSL-2.


   TORNADO platform:
   -----------------
        This demo compiles for TORNADO-6x, TORNADO-P6x and TORNADO-E6x
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. MIRAGE JTAG emulator and TI CCS tools are
	required to recompile this demo and to upload code to the
	DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration jumpers:
          - no interrupt from UART
	  - no interrupt from DIO

***********************************************************************/

#define	_E6X_	1
#define	_T6X_	2
#define	_TP6X_	3

//!!!! change this constant in order to recompile for specific TORNADO
#define	_TORNADO_	_TP6X_

#define	_E6X_SELECTED	(_TORNADO_==_E6X_)
#define	_T6X_SELECTED	(_TORNADO_==_T6X_)
#define	_TP6X_SELECTED	(_TORNADO_==_TP6X_)

#include	"math.h"
#include	"c6x_def.h"
#include	"ddc1_dsp.h"


#define	_WAIT_SIO_TRDY		while(!((*C6X_MCBSP0_SPCR) & 0x20000))
#define	_WAIT_SIO_RRDY		while(!((*C6X_MCBSP0_SPCR) & 0x2))
#define	_WRITE_SIO_DATA		(*C6X_MCBSP0_DXR)
#define _READ_SIO_DATA		(*C6X_MCBSP0_DRR)
#define	_CLEAR_TM0_EVENT	ICR=0x4000
#define	_WAIT_TM0_EVENT		while(!(IFR & 0x4000))


//------ TORNADO platform specific fucntions --------------------
#if(_E6X_SELECTED)
#define _REMOVE_SIOX_RESET	TE6X_set_pxsx_run(0x3)
#include	"te6x_dsp.h"
#endif

#if(_T6X_SELECTED)
#define _REMOVE_SIOX_RESET	T6X_set_pxsx_run(0x7)
#include	"t6x_dsp.h"
#endif

#if(_TP6X_SELECTED)
#define _REMOVE_SIOX_RESET	TP6X_set_pxsx_run(0x7)
#include	"tp6x_dsp.h"
#endif
//--------------------------------------------------------------


	//--- writes DATA to DAC1
void write_dac1(data)
unsigned data;
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send DAC-1 write command
	_WRITE_SIO_DATA =(   TSDAS_DDC1_DAC1_WR_CMND
			   | ((data<<TSDAS_DDC1_DAC_DATA_SHIFT) & TSDAS_DDC1_DAC_DATA_BITMASK)
			 )  ;
}

	//--- writes DATA to DAC2
void write_dac2(data)
unsigned data;
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send DAC-2 write command
	_WRITE_SIO_DATA =(   TSDAS_DDC1_DAC2_WR_CMND
			   | ((data<<TSDAS_DDC1_DAC_DATA_SHIFT) & TSDAS_DDC1_DAC_DATA_BITMASK)
			 )  ;
}


void set_dio_output(data)
unsigned data;
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send DIO write command
	_WRITE_SIO_DATA =(   TSDAS_DDC1_DIO_WR_CMND
			   | (  TSDAS_DDC1_DIO0_DIR_OUT
			      | TSDAS_DDC1_DIO1_DIR_OUT
			      | ((data==0) ? (TSDAS_DDC1_DIO1_DATA1 | TSDAS_DDC1_DIO0_DATA0):
					     (TSDAS_DDC1_DIO1_DATA0 | TSDAS_DDC1_DIO0_DATA1)
				)
                             )
			 )  ;
}


void set_xsl_fmt()
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send XSL FMT command
	_WRITE_SIO_DATA =(   TSDAS_DDC1_XSL_SET_FMT_CMND
			   | TSDAS_DDC1_XSL_FMT_WLEN32
			   | TSDAS_DDC1_XSL_FMT_CLKPOL_POS
			   | TSDAS_DDC1_XSL_FMT_CLKFRM_ON
			 )  ;
}


void set_xsl1(data)
unsigned data;
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send XSL1 write command
	_WRITE_SIO_DATA =(   TSDAS_DDC1_XSL1_WR_CMND
			   | (data & TSDAS_DDC1_XSL_DATA32_BITMASK)
			 )  ;
}



void set_xsl2(data)
unsigned data;
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send XSL2 write command
	_WRITE_SIO_DATA =(   TSDAS_DDC1_XSL2_WR_CMND
			   | (data & TSDAS_DDC1_XSL_DATA32_BITMASK)
			 )  ;
}



void init_cpu()
 {
	 // assume that C6x EMIF is initialized as default on reset
 }

void init_sio()
{
		//-- configure C6x McBSP-0 port
	*C6X_MCBSP0_SPCR = TSDAS_DDC1_C6X_MCBSP_SPCR_DATA;
	*C6X_MCBSP0_PCR  = TSDAS_DDC1_C6X_MCBSP_PCR_DATA;
	*C6X_MCBSP0_XCR  = TSDAS_DDC1_C6X_MCBSP_XRCR_DATA;
	*C6X_MCBSP0_RCR  = TSDAS_DDC1_C6X_MCBSP_XRCR_DATA;
	*C6X_MCBSP0_SRGR = TSDAS_DDC1_C6201_MCBSP_SRGR_DATA;
	*C6X_MCBSP0_SPCR |= TSDAS_DDC1_C6X_MCBSP_SPCR_GO_MASK;

		//--- set 25 kHz sampling frequency for DACs
	*C6X_TM0_CONTROL = 0;
	*C6X_TM0_PERIOD  = 2000;
	*C6X_TM0_CONTROL = 0x03c1;	// start timer

        	//--- clear TIMER-0 interrupt request (INT-14)
	_CLEAR_TM0_EVENT;
	_REMOVE_SIOX_RESET;
}





void main()
{  int i;
   float dac1, dac2;

	init_cpu();
	init_sio();
	set_xsl_fmt();

        i=0;

	while (1)
	 {
	  _WAIT_TM0_EVENT;
	  _CLEAR_TM0_EVENT;

          dac1 = 2040.0 * (1.0 + sin(2.0*3.14*((float)i)/1000.0));
          dac2 = 1200.0 * (1.0 + cos(4.0*3.14*((float)i)/1000.0));

	  write_dac1((int) dac1);
	  write_dac2((int) dac2);
	  set_dio_output(i%2);

		// send test patterns to XSL-1 and XSL-2
          set_xsl1(0x20000001);
          set_xsl2(0x2aaaaaaa);


          i++;
	  if(i>=1000) i=0;
	 }


}

