/***********************************************************************
	T/SDAS-DDC1 DCM Demo Utility. PDC Demo. v.1A
	(C) MicroLAB Systems, 1999-2000


   Description:
   ------------
	This demo utilizes PDC chip and provides real-time output of
	AM demodulated signal and FM demodulated signal via
	on-board DAC-1/DAC-2 chips. The output of DAC-1 is
	proportional to the magnitude of input RF_in input and demonstrates
	AM demodulation feature of HSP50214 PDC. The output of DAC-2
	is proportional	to the frequency of input signal and demonstrates
	FM demodulation feature of HSP50214 PDC.

   TORNADO platform:
   -----------------
        This demo compiles for TORNADO-6x, TORNADO-P6x and TORNADO-E6x
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. MIRAGE JTAG emulator and TI CCS tools are
	required to recompile this demo and to upload code to the
	DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
	RF_in frequency = 1MHz +-40kHz  
	RF_in amplitude = 0.4 V

        DAC-1 output:  proportional to RF_in magnitude
                       (can be used for AM demodulation)

	DAC-2 output:  proportional to the frequency of
                       (RF_in-1MHz) signal
                       (can be used for FM demodulation)

	PDC output FIR filter:
	      filter type:	LPF, 251-th order
	         bandpass:	<40kHz
	         bandstop:     	>42kHz
	  bandstop ripple: 	-60dB
	  bandpass ripple: 	<1dB

        on-board configuration jumpers:
          - no interrupt from UART
	  - no interrupt from DIO
	  - on-board Fs/CLKIN 60MHz oscillator is used
	  - on-board PDC PROCLK  50MHz oscillator is used

***********************************************************************/

#define	_E6X_	1
#define	_T6X_	2
#define	_TP6X_	3

//!!!! change this constant in order to recompile for specific TORNADO
#define	_TORNADO_	_E6X_

#define	_E6X_SELECTED	(_TORNADO_==_E6X_)
#define	_T6X_SELECTED	(_TORNADO_==_T6X_)
#define	_TP6X_SELECTED	(_TORNADO_==_TP6X_)

#define _TI_ENHANCED_MATH_H 1
#include	"math.h"
#include	<stdlib.h>

#include	"c6x_def.h"
#include	"ddc1_dsp.h"

//------ TORNADO platform specific fucntions --------------------
#if(_E6X_SELECTED)
#define _REMOVE_SIOX_RESET	TE6X_set_pxsx_run(0x3)
#include	"te6x_dsp.h"
#endif

#if(_T6X_SELECTED)
#define _REMOVE_SIOX_RESET	T6X_set_pxsx_run(0x7)
#include	"t6x_dsp.h"
#endif

#if(_TP6X_SELECTED)
#define _REMOVE_SIOX_RESET	TP6X_set_pxsx_run(0x7)
#include	"tp6x_dsp.h"
#endif
//--------------------------------------------------------------


//~~~~~~~~~~~ HSP50214 PDC uploadable configuration data ~~~~~~~~~~

int hsp50214B_init[256]=
{
0x0000a708,  0x00010000,  0x00000000,  0x04444444,
0x00000000,  0x00000000,  0x00000000,  0x000f8efb,
0x00000000,  0x0b000b00,  0x00000000,  0x00000000,
0xcccccccc,  0x00000000,  0x00000000,  0x00000000,
0x00000a61,  0x00004a80,  0x00000000,  0x0a254800,
0x000b0000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000278,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
/* Discriminator FIR Coefficients */
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x001fffff,  0x001fffff,  0x001fffff,  0x001fffff,
0x001fffff,  0x001fffff,  0x001fffff,  0x001fffff,
0x001fffff,  0x001fffff,  0x001fffff,  0x001fffff,
0x001fffff,  0x001fffff,  0x001fffff,  0x001fffff,
0x001fffff,  0x001fffff,  0x001fffff,  0x001fffff,
0x001fffff,  0x001fffff,  0x001fffff,  0x001fffff,
0x001fffff,  0x001fffff,  0x001fffff,  0x001fffff,
0x001fffff,  0x001fffff,  0x001fffff,  0x001fffff,
/* Programmable FIR Coefficients */
0x0014fdf3,  0x0008fc3c,  0xfffbc5de,  0xffffa8e9,
0x0002594a,  0xfffe7000,  0xffffa9c7,  0x000161de,
0xffff1da5,  0xffffab35,  0x0000fb7c,  0xffff6e79,
0xffffad2d,  0x0000c1b4,  0xffff9dd4,  0xffffafa5,
0x00009b78,  0xffffbd28,  0xffffb294,  0x00007f98,
0xffffd36a,  0xffffb5ee,  0x000069f0,  0xffffe3ef,
0xffffb9a3,  0x00005860,  0xfffff083,  0xffffbda6,
0x000049b6,  0xfffffa35,  0xffffc1e7,  0x00003d3b,
0x000001b1,  0xffffc654,  0x0000327a,  0x0000076e,
0xffffcadf,  0x00002927,  0x00000bbe,  0xffffcf76,
0x0000210e,  0x00000ee2,  0xffffd40a,  0x00001a06,
0x0000110b,  0xffffd88c,  0x000013f3,  0x00001263,
0xffffdcef,  0x00000ebd,  0x0000130c,  0xffffe128,
0x00000a50,  0x00001325,  0xffffe52a,  0x00000699,
0x000012ca,  0xffffe8ef,  0x00000387,  0x00001211,
0xffffec6e,  0x0000010a,  0x00001111,  0xffffefa3,
0xffffff15,  0x00000fdc,  0xfffff28b,  0xfffffd96,
0x00000e84,  0xfffff523,  0xfffffc7f,  0x00000d17,
0xfffff76d,  0xfffffbc3,  0x00000ba3,  0xfffff969,
0xfffffb53,  0x00000a31,  0xfffffb1b,  0xfffffb23,
0x000008cc,  0xfffffc87,  0xfffffb27,  0x0000077a,
0xfffffdb1,  0xfffffb53,  0x00000640,  0xfffffea0,
0xfffffb9e,  0x00000523,  0xffffff5a,  0xfffffbfe,
0x00000425,  0xffffffe4,  0xfffffc6b,  0x00000346,
0x00000046,  0xfffffcde,  0x00000286,  0x00000086,
0xfffffd53,  0x000001e5,  0x000000ab,  0xfffffdc3,
0x00000161,  0x000000ba,  0xfffffe2d,  0x000000f7,
0x000000b9,  0xfffffe8d,  0x000000a4,  0x000000ab,
0xfffffee2,  0x00000067,  0x00000096,  0xffffff2b,
0x0000003a,  0x0000007c,  0xffffff67,  0x0000001c,
0x00000061,  0xffffff99,  0x00000009,  0x00000047,
0xffffffbf,  0x00000000,  0x00000000,  0x00000000

};

short *hsp_data=(short *)0x10000;
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~




void	cycle_pause()
{
	*C6X_TM1_CONTROL =0x301;
	*C6X_TM1_PERIOD  =1024;
	*C6X_TM1_CONTROL |=0xc0;
	while(*C6X_TM1_COUNTER <1000) ;
	*C6X_TM1_CONTROL=0;
}

void init_sio()
{
/* SPCR: 00000000 0x00 0000 0000 0000 0000 0000 */
	*C6X_MCBSP0_SPCR = 0x00000000;

/* PCR:  0000000000000000 0000 1010 0000 1111 */
	*C6X_MCBSP0_PCR = 0x00000a0e;

/* TCR:  00000000 0100 0001 00000000 0100 0000 */
	*C6X_MCBSP0_XCR = 0x00410040;

/* RCR:  00000001 1010 0001 00000001 1010 0000 2 words per 32 bits */
	*C6X_MCBSP0_RCR = 0x01a101a0;

/* SRGR: 0011 0000 0000 0000 0001 0000 0100 0000 */
	*C6X_MCBSP0_SRGR = 0x20401050;

	cycle_pause();
	*C6X_MCBSP0_SPCR |= 0x00400000;
	cycle_pause();
	*C6X_MCBSP0_SPCR |= 0x00010001;
	cycle_pause();
	*C6X_MCBSP0_DXR = 0;	// load McBSP-0 XMIT-dataRG with dummy data
}

cpu_init()
{
	if(   (C6X_INTMUX_SET(4,C6X_INTSEL_TINT0)!=0) ||
		  (C6X_INTMUX_SET(5,C6X_INTSEL_XINT7)!=0)
		) return(-1);
	C6X_XINT_ALL_POLARITY_SET(C6X_XINT_POLARITY_NEG);
	C6X_INT_DISABLE_ALL;
	return(0);
}

void init_DDC(int *data,int addr,int len)
{
	volatile int i,j,k,l,dat;
	j=addr;
	k=addr;
	l=len;

    rep0:
	for(i=0;i<l;i++) {
		dat=data[j++];
		*C6X_MCBSP0_DXR = (dat&0xff)|0x3800;dat>>=8;
		cycle_pause();
		*C6X_MCBSP0_DXR = (dat&0xff)|0x3900;dat>>=8;
		cycle_pause();
		*C6X_MCBSP0_DXR = (dat&0xff)|0x3a00;dat>>=8;
		cycle_pause();
		*C6X_MCBSP0_DXR = (dat&0xff)|0x3b00;dat>>=8;
		cycle_pause();
		*C6X_MCBSP0_DXR = (k&0xff)|0x3c00;
		cycle_pause();
		k++;
		}
	if(l==28) {
		l=25;
		k=0;
		j=0;
		goto rep0;
		}
	if(addr==0) {
		*C6X_MCBSP0_DXR = 0x4a01;
		cycle_pause();
		*C6X_MCBSP0_DXR = 0x4900;
		cycle_pause();
		}
}

void out_MAG(int mag_level)
{
	*C6X_MCBSP0_DXR = (mag_level&0xfff)|0x0000;
	cycle_pause();
}

void out_FREQ(int freq_level)
{
	*C6X_MCBSP0_DXR = (freq_level &0xfff)|0x1000;
	cycle_pause();
}


void wait_DMA(void)
{
	while((*C6X_DMA0_PCRG &0xc)!=0) ;
}

void init_DMA(int n_frame, int frame_length)
{
	*C6X_MCBSP0_SPCR &= 0xffbefffe; /* Stop MCBSP0 */
	*C6X_DMA0_SARG=(unsigned int)0x018C0000; /* Data from MCBSP0 */
	*C6X_DMA0_GACRG=0x018C0000; /* Source address reload */
	*C6X_DMA0_DARG=0x10000;
	*C6X_DMA0_GADRG=0x10000; /* Destination address reload */
	*C6X_DMA0_TCRG=4;
	*C6X_DMA0_GCRARG=4; /* Count reload */

/* 11100001  0000 0  011 01 00 0000 0100 0000 */
/*
   11 - DST reload (use DMA global address register D)
   10 - SRC reload (use DMA global address register C)
   0 - EMOD (DMA running during an emulation halt)
   0 - FS (Frame synchronization disable)
   0 - TCINT (interrupt disabled)
   1 - PRI (DMA priority)       
   00000 - WSYNC (no synchronization)
   01101 - RSYNC (MCSP0 receive event)
   0 - INDEX (use DMA global index register 2)
   0 - CNT reload (reload with DMA global count reload register 0)
   00 - SPLIT (split channel mode disabled)
   00 - ESIZE (element size=32-bit)
   01 - DST DIR (increment by element size in bytes)
   00 - SRC DIR (no modification)
   00 - STATUS
   00 - START
*/
	*C6X_DMA0_PCRG=0xe1034040;

/* 000000000000 0001 1010000000000000 */
	*C6X_DMA0_SCRG=0x00010000;
	*C6X_DMA0_PCRG|=1;
	*C6X_MCBSP0_SPCR |= 0x00410001; /* Start MCBSP0 */
	wait_DMA();

	*C6X_MCBSP0_SPCR &= 0xffbefffe; /* Stop MCBSP0 */
	cycle_pause();

/* 11100001  0000 0  011 01 00 0000 0100 0000 */
	*C6X_DMA0_PCRG=0xe1034040;
	*C6X_DMA0_DARG=0x10000;
	*C6X_DMA0_GADRG=0x10000; // Destination address reload
	*C6X_DMA0_TCRG=(n_frame<<16)+frame_length;
	*C6X_DMA0_GCRARG=(n_frame<<16)+frame_length; // Count reload

	*C6X_DMA0_SCRG=0x0001a000; /* Clear synchronization status */
	cycle_pause();
	*C6X_DMA0_PCRG|=3;
	*C6X_MCBSP0_SPCR |= 0x00410001; /* Start MCBSP0 */
}

void main()
{
int i;                
short data;
volatile int old_ptr;

	if( cpu_init()!=0) while(1);	// idle here in incorrect DSP init

	C6X_INT_CLEAR_ALL;
	C6X_GINT_ENABLE;		/* set GIE bit */
	_REMOVE_SIOX_RESET;

	init_sio();
	C6X_CACHE_ENABLE;

	*C6X_TM0_CONTROL     = 0x200;
	*C6X_TM0_PERIOD      = 15000;
	*C6X_TM0_COUNTER     = 0;
	*C6X_TM0_CONTROL     |= 0xc0;

	init_DDC(hsp50214B_init,0,28);
	init_DDC(hsp50214B_init,0,28);
	init_DDC(hsp50214B_init,64,192);
	init_DDC(hsp50214B_init,64,192);
                                                       
	init_DMA(1,8192);    
	old_ptr=0x10000;

	while(1) {
		i=*C6X_DMA0_DARG;               
		i&=0xfffffff8; /* 4 16-bits word */
		if(i!=old_ptr) {
			old_ptr=i;
			i-=0x10000; 
			i>>=1;
/* MAG & FREQ out */		
		data=*(hsp_data+i+2);
		data+=2048;
		out_FREQ(data);
		data=*(hsp_data+i+3);
		data>>=4;
		out_MAG(data);

/* I & Q out */		
/*		
		data=*(hsp_data+i);
		data>>=4;
		data+=2048;
		out_FREQ(data);
		data=*(hsp_data+i+1);
		data>>=4;
		data+=2048;
		out_MAG(data);
*/
		}
	}
}
