/********************************************************************
   T/SDAS-DDC1 DCM Demo Utility. UART demo for C6x. rev.1A
   (C) MicroLAB Systems, 1999


   Description:
   ------------
	This demo utilizes UART chip and provides communication
	with any external RS232C erminal at 9600 bauds. This demo
	just returns the received symbols back to terminal.

   TORNADO platform:
   -----------------
        This demo compiles for TORNADO-6x, TORNADO-P6x and TORNADO-E6x
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. MIRAGE JTAG emulator and TI CCS tools are
	required to recompile this demo and to upload code to the
	DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
	baud rate: 	9600
	word length:	8 bits
	stop bits: 	2

        on-board configuration jumpers:
          - no interrupt from UART
	  - no interrupt from DIO

***********************************************************************/

#define	_E6X_	1
#define	_T6X_	2
#define	_TP6X_	3

//!!!! change this constant in order to recompile for specific TORNADO
#define	_TORNADO_	_E6X_

#define	_E6X_SELECTED	(_TORNADO_==_E6X_)
#define	_T6X_SELECTED	(_TORNADO_==_T6X_)
#define	_TP6X_SELECTED	(_TORNADO_==_TP6X_)


#include	"c6x_def.h"
#include	"ddc1_dsp.h"


char welcome[]	= "T/SDAS-DDC1 welcomes you!\n\n>>";
char cts_low[]	= "<< RTS: CTS=LOW >>";
char cts_high[]	= "<< RTS: CTS=HIGH >>";
char dsr_low[]	= "<< DTR: DSR=LOW >>";
char dsr_high[]	= "<< DTR: DSR=HIGH >>";

#define	_WAIT_SIO_TRDY		while(!((*C6X_MCBSP0_SPCR) & 0x20000))
#define	_WAIT_SIO_RRDY		while(!((*C6X_MCBSP0_SPCR) & 0x2))
#define	_WRITE_SIO_DATA		(*C6X_MCBSP0_DXR)
#define _READ_SIO_DATA		(*C6X_MCBSP0_DRR)

//------ TORNADO platform specific fucntions --------------------
#if(_E6X_SELECTED)
#define _REMOVE_SIOX_RESET	TE6X_set_pxsx_run(0x3)
#include	"te6x_dsp.h"
#endif

#if(_T6X_SELECTED)
#define _REMOVE_SIOX_RESET	T6X_set_pxsx_run(0x7)
#include	"t6x_dsp.h"
#endif

#if(_TP6X_SELECTED)
#define _REMOVE_SIOX_RESET	TP6X_set_pxsx_run(0x7)
#include	"tp6x_dsp.h"
#endif
//--------------------------------------------------------------



void write_uart(addr,data)
	// writes to UART register #ADDR
unsigned addr,data;
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send UART write command
	_WRITE_SIO_DATA =(   TSDAS_DDC1_UART_WR_CMND
                           | ((addr<<TSDAS_DDC1_UART_ADDR_SHIFT) & TSDAS_DDC1_UART_ADDR_BITMASK)
			   | ((data<<TSDAS_DDC1_UART_DATA_SHIFT) & TSDAS_DDC1_UART_DATA_BITMASK)
			 )  ;
}



unsigned read_uart(unsigned addr)
	// reads UART register #ADDR
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send UART read command
	_WRITE_SIO_DATA =(   TSDAS_DDC1_UART_RD_CMND
                           | ((addr<<TSDAS_DDC1_UART_ADDR_SHIFT) & TSDAS_DDC1_UART_ADDR_BITMASK)
			 )  ;

		//-- wait for McBSP receiver is ready
	_WAIT_SIO_RRDY;

	return((_READ_SIO_DATA>>TSDAS_DDC1_UART_DATA_SHIFT) & 0xff);
}


uart_transmit_char(c)
int c;
{
  while(!(read_uart(TSDAS_DDC1_UART_LSTAT_RG) & TSDAS_DDC1_UART_LSTAT_THEMPY));
  write_uart(TSDAS_DDC1_UART_TDATA_RG,c);
}



unsigned uart_receive_char()
{   unsigned d;

  do
    { d = read_uart(TSDAS_DDC1_UART_LSTAT_RG);
    }
     while(!(d & TSDAS_DDC1_UART_LSTAT_DR));

  return(read_uart(TSDAS_DDC1_UART_RDATA_RG));
}


int uart_receiver_status()
{
  return(read_uart(TSDAS_DDC1_UART_LSTAT_RG) & TSDAS_DDC1_UART_LSTAT_DR);
}


void init_uart()
 {
		// configure UART for 9600/8/N/2

	   write_uart(TSDAS_DDC1_UART_LCNTR_RG,
			TSDAS_DDC1_UART_LCNTR_DLAB
		      | TSDAS_DDC1_UART_LCNTR_WLEN8
		      | TSDAS_DDC1_UART_LCNTR_STOP2
		     );

	   write_uart(TSDAS_DDC1_UART_DIVL_RG,TSDAS_DDC1_UART_DIVL_9600);
	   write_uart(TSDAS_DDC1_UART_DIVH_RG,TSDAS_DDC1_UART_DIVH_9600);

	   write_uart(TSDAS_DDC1_UART_LCNTR_RG,
		        TSDAS_DDC1_UART_LCNTR_WLEN8
		      | TSDAS_DDC1_UART_LCNTR_STOP2
		     );

	   write_uart(TSDAS_DDC1_UART_MCNTR_RG,0);
	   write_uart(TSDAS_DDC1_UART_IE_RG,TSDAS_DDC1_UART_IE_DR);
 }



void uart_transmit_mess(s)
char *s;
{
	  while((*s &0xff)!=0) uart_transmit_char(*(s++));
}


void init_cpu()
 {
	 // assume that C6x EMIF is initialized as default on reset
 }

void init_sio()
{
		//-- configure C6x McBSP-0 port
	*C6X_MCBSP0_SPCR = TSDAS_DDC1_C6X_MCBSP_SPCR_DATA;
	*C6X_MCBSP0_PCR  = TSDAS_DDC1_C6X_MCBSP_PCR_DATA;
	*C6X_MCBSP0_XCR  = TSDAS_DDC1_C6X_MCBSP_XRCR_DATA;
	*C6X_MCBSP0_RCR  = TSDAS_DDC1_C6X_MCBSP_XRCR_DATA;
	*C6X_MCBSP0_SRGR = TSDAS_DDC1_C6201_MCBSP_SRGR_DATA;
	*C6X_MCBSP0_SPCR |= TSDAS_DDC1_C6X_MCBSP_SPCR_GO_MASK;

	_REMOVE_SIOX_RESET;
}





void main()
{
   int modem_status;

	init_cpu();
	init_sio();
	init_uart();

        uart_transmit_mess(welcome);


     while (1)
	{
	 modem_status=read_uart(TSDAS_DDC1_UART_MSTAT_RG);

		// copy CTS->RTS
	 if(modem_status & TSDAS_DDC1_UART_MSTAT_CTS)  
	     write_uart(TSDAS_DDC1_UART_MCNTR_RG,(read_uart(TSDAS_DDC1_UART_MCNTR_RG) | TSDAS_DDC1_UART_MCNTR_RTS));
	   else
	     write_uart(TSDAS_DDC1_UART_MCNTR_RG,(read_uart(TSDAS_DDC1_UART_MCNTR_RG) & ~TSDAS_DDC1_UART_MCNTR_RTS));

		// copy DSR->DTR
	 if(modem_status & TSDAS_DDC1_UART_MSTAT_DSR)  
	     write_uart(TSDAS_DDC1_UART_MCNTR_RG,(read_uart(TSDAS_DDC1_UART_MCNTR_RG) | TSDAS_DDC1_UART_MCNTR_DTR));
	   else
	     write_uart(TSDAS_DDC1_UART_MCNTR_RG,(read_uart(TSDAS_DDC1_UART_MCNTR_RG) & ~TSDAS_DDC1_UART_MCNTR_DTR));


		// detect CTS change
   	 if(modem_status & TSDAS_DDC1_UART_MSTAT_DCTS)  
	    {
	      if(modem_status & TSDAS_DDC1_UART_MSTAT_CTS)
		   uart_transmit_mess(cts_high);
		else
		   uart_transmit_mess(cts_low);
	    }

		// detect DSR change
	 if(modem_status & TSDAS_DDC1_UART_MSTAT_DDSR)
	    {
	      if(modem_status & TSDAS_DDC1_UART_MSTAT_DSR)
		   uart_transmit_mess(dsr_high);
		else
		   uart_transmit_mess(dsr_low);
	    }


	 if(uart_receiver_status()==TSDAS_DDC1_UART_LSTAT_DR)
              uart_transmit_char(uart_receive_char());
	 }

}

